// Dart imports:
import 'dart:async';

// Flutter imports:
import 'package:flutter/cupertino.dart';

// Project imports:
import 'package:zego_uikit/zego_uikit.dart';
import 'service_p.dart';

mixin ZegoUIKitUserInRoomAttributesPluginService {
  final _private = ZegoUIKitUserInRoomAttributesPluginServicePrivate();

  void initUserInRoomAttributes() {
    ZegoLoggerService.logInfo(
      "user in-room init",
      tag: "uikit",
      subTag: "user attributes",
    );
    _private.init();
  }

  void uninitUserInRoomAttributes() {
    ZegoLoggerService.logInfo(
      "user in-room uninit",
      tag: "uikit",
      subTag: "user attributes",
    );
    _private.uninit();
  }

  /// set users in-room attributes
  Future<ZegoSignalingPluginResult> setUsersInRoomAttributes(
    String key,
    String value,
    List<String> userIDs,
  ) async {
    Map result = await ZegoUIKit()
        .getPlugin(ZegoUIKitPluginType.signaling)!
        .invoke('setUsersInRoomAttributes', {
      'key': key,
      'value': value,
      'userIDs': userIDs,
    });

    return ZegoSignalingPluginResult(
      result["errorCode"] as String,
      result["errorMessage"] as String,
      result: result['errorUserList'] as List<String>,
    );
  }

  /// query user in-room attributes
  Future<ZegoSignalingPluginResult> queryUsersInRoomAttributes({
    String nextFlag = '',
    int count = 100,
  }) async {
    Map pluginResult = await ZegoUIKit()
        .getPlugin(ZegoUIKitPluginType.signaling)!
        .invoke('queryUsersInRoomAttributes', {
      'nextFlag': nextFlag,
      'count': count,
    });

    var infos = pluginResult['infos'] as Map<String, Map<String, String>>;
    var result = ZegoSignalingPluginResult(
      pluginResult["errorCode"] as String,
      pluginResult["errorMessage"] as String,
      result: infos,
    );

    _private.updateUserInRoomAttributes(infos);

    return result;
  }

  /// get users in-room attributes notifier
  Stream<ZegoSignalingUserInRoomAttributesData>
      getUsersInRoomAttributesStream() {
    return ZegoUIKit()
        .getPlugin(ZegoUIKitPluginType.signaling)!
        .getEventStream('usersInRoomAttributes')
        .map((data) {
      return ZegoSignalingUserInRoomAttributesData(
        editor: data["editor"] as ZegoUIKitUser?,
        infos: data["infos"] as Map<String, Map<String, String>>,
      );
    });
  }
}
