// Dart imports:
import 'dart:async';

// Flutter imports:
import 'package:flutter/cupertino.dart';

// Project imports:
import 'package:zego_uikit/zego_uikit.dart';

class ZegoUIKitUserInRoomAttributesPluginServicePrivate {
  final bool isInit = false;
  final List<StreamSubscription<dynamic>?> subscriptions = [];
  final Map<String, Map<String, String>> pendingUserRoomAttributes = {};

  void init() {
    if (isInit) {
      ZegoLoggerService.logInfo(
        "user in-room private had init",
        tag: "uikit",
        subTag: "user attributes",
      );
      return;
    }

    ZegoLoggerService.logInfo(
      "user in-room private init",
      tag: "uikit",
      subTag: "user attributes",
    );
    subscriptions
      ..add(ZegoUIKit().getUserListStream().listen(_onUserListUpdated))
      ..add(ZegoUIKit()
          .getSignalingPlugin()
          .getUsersInRoomAttributesStream()
          .listen(onUsersAttributesUpdated));
  }

  void uninit() {
    if (!isInit) {
      ZegoLoggerService.logInfo(
        "user in-room private is not init",
        tag: "uikit",
        subTag: "user attributes",
      );
      return;
    }

    ZegoLoggerService.logInfo(
      "user in-room private uninit",
      tag: "uikit",
      subTag: "user attributes",
    );
    for (var subscription in subscriptions) {
      subscription?.cancel();
    }
  }

  void onUsersAttributesUpdated(
      ZegoSignalingUserInRoomAttributesData attributesData) {
    updateUserInRoomAttributes(attributesData.infos);
  }

  void _onUserListUpdated(List<ZegoUIKitUser> users) {
    var doneUserIDs = <String>[];
    pendingUserRoomAttributes.forEach((userID, userAttributes) {
      ZegoLoggerService.logInfo(
        "exist pending user attribute, user id: $userID, attributes: $userAttributes",
        tag: "uikit",
        subTag: "user attributes",
      );

      var user = ZegoUIKit().getUser(userID);
      if (user != null && !user.isEmpty()) {
        updateUserInRoomAttributes({userID: userAttributes});

        doneUserIDs.add(userID);
      }
    });

    pendingUserRoomAttributes
        .removeWhere((userID, userAttributes) => doneUserIDs.contains(userID));
  }

  void updateUserInRoomAttributes(Map<String, Map<String, String>> infos) {
    infos.forEach((updateUserID, updateUserAttributes) {
      var updateUser = ZegoUIKit().getUser(updateUserID);
      if (null == updateUser) {
        pendingUserRoomAttributes[updateUserID] = updateUserAttributes;
        return;
      }

      ZegoUIKit().getInRoomUserAttributesNotifier(updateUserID).value =
          updateUserAttributes;
      ZegoLoggerService.logInfo(
        "user in-room attributes update, user id:$updateUserID, attributes:$updateUserAttributes",
        tag: "uikit",
        subTag: "user attributes",
      );
    });
  }
}
